#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Mar  3 22:46:28 2022

@author: anton
"""

from SLiCAP import *

fileName = "vAmpNoise1"

prj = initProject(fileName)

# Specification

Rs = 600        # Source resistance
F  = 2          # Noise figure in dB
Av = 10         # Voltage gain
NF = 10**(F/10) # Noise figure

#makeNetlist(fileName + "_design.asc", fileName + "_design")
i1 = instruction()
i1.setCircuit(fileName + ".cir")

i1.defPar("R_s", Rs)

htmlPage("Circuit data")
head2html("Circuit diagram")
img2html(fileName + ".svg", 600)
netlist2html(fileName + ".cir")
elementData2html(i1.circuit)
params2html(i1.circuit)

# Determine the gain
i1.setSimType("numeric")
i1.setGainType("gain")
i1.setDataType("laplace")
i1.setSource("V1")
i1.setDetector("V_out")

gain = i1.execute().laplace

# Assign a value of R1 such that the gain equals Av
# The result is a function of R_2

R1 = sp.solve(gain-Av, sp.Symbol("R_1"))[0]

# Assign this value to the circuit parameter 'R_1'
i1.defPar("R_1", R1)

# Now calculate the noise figure (it will be a function of R_2)
i1.setGainType("vi")
i1.setDataType("noise")
noiseResult = i1.execute()

noiseFigure = noiseResult.inoise/noiseResult.inoiseTerms["I_noise_R3"]

# Solve R_2 for the maximum value of the noise figure
R2 = sp.solve(NF-noiseFigure, sp.Symbol("R_2"))[0]

htmlPage("Show-stopper values $R_1$, $R_2$")
text2html("The maximum values of R1 and R2 are:")

# Substitute the value of R2 in the expression of R1
R1 = R1.subs(sp.Symbol("R_2"), R2)

# Print the results on the HTML page
eqn2html("R_1", R1)
eqn2html("R_2", R2)

################################# Verification ################################
    
# Found from hand calculations:
R1 = 3510
R2 = 390

# Assign the numeric values to the circuit parameters
i1.defPar("R_s", Rs)
i1.defPar("R_1", R1)
i1.defPar("R_2", R2)

# Verify the hand calculations
htmlPage("Noise verification")

i1.setSimType("numeric")
i1.setGainType("vi")
i1.setDataType("noise")
i1.setSource("V1")
i1.setDetector("V_out")
noiseResult = i1.execute()

# Calculate the noise figure
NF = noiseResult.inoise/noiseResult.inoiseTerms['I_noise_R3']
F = sp.N(10*sp.log(NF)/sp.log(10))
head2html("Noise figure")
eqn2html("F", F, units="dB")

# Calculate the voltage gain
Av = 1 + R1/R2
head2html("Voltage gain")
eqn2html("A_v", Av)